'use strict';

var _utilHelpers = require('./util/helpers');

(0, _utilHelpers.test)('attribute selector', '[href]', function (t, tree) {
    t.plan(2);
    t.equal(tree.selectors[0].rules[0].attribute, 'href');
    t.equal(tree.selectors[0].rules[0].type, 'attribute');
});

(0, _utilHelpers.test)('multiple attribute selectors', '[href][class][name]', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].attribute, 'href');
    t.equal(tree.selectors[0].rules[1].attribute, 'class');
    t.equal(tree.selectors[0].rules[2].attribute, 'name');
});

(0, _utilHelpers.test)('attribute selector with a value', '[name=james]', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].attribute, 'name');
    t.equal(tree.selectors[0].rules[0].operator, '=');
    t.equal(tree.selectors[0].rules[0].value, 'james');
});

(0, _utilHelpers.test)('attribute selector with quoted value', '[name="james"]', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].attribute, 'name');
    t.equal(tree.selectors[0].rules[0].operator, '=');
    t.equal(tree.selectors[0].rules[0].value, '"james"');
});

(0, _utilHelpers.test)('multiple attribute selectors + combinator', '[href][class][name] h1 > h2', function (t, tree) {
    t.plan(4);
    t.equal(tree.selectors[0].rules[2].attribute, 'name');
    t.equal(tree.selectors[0].rules[2].combinator, ' ');
    t.equal(tree.selectors[0].rules[2].rules[0].combinator, ' > ');
    t.equal(tree.selectors[0].rules[2].rules[0].rules[0].value, 'h2');
});

(0, _utilHelpers.test)('attribute, class, combinator', '[href] > h2.test', function (t, tree) {
    t.plan(4);
    t.equal(tree.selectors[0].rules[0].attribute, 'href');
    t.equal(tree.selectors[0].rules[0].combinator, ' > ');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'h2');
    t.equal(tree.selectors[0].rules[0].rules[1].value, 'test');
});

(0, _utilHelpers.test)('attribute selector with quoted value & combinator', '[name="james"] > h1', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].value, '"james"');
    t.equal(tree.selectors[0].rules[0].combinator, ' > ');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'h1');
});

(0, _utilHelpers.test)('multiple quoted attribute selectors', '[href*="test.com"][rel="external"][id][class~="test"] > [name]', function (t, tree) {
    t.plan(11);
    t.equal(tree.selectors[0].rules[0].attribute, 'href');
    t.equal(tree.selectors[0].rules[0].value, '"test.com"');
    t.equal(tree.selectors[0].rules[1].attribute, 'rel');
    t.equal(tree.selectors[0].rules[1].value, '"external"');
    t.equal(tree.selectors[0].rules[2].attribute, 'id');
    t.notOk(tree.selectors[0].rules[2].value, 'should not have a value');
    t.equal(tree.selectors[0].rules[3].attribute, 'class');
    t.equal(tree.selectors[0].rules[3].value, '"test"');
    t.equal(tree.selectors[0].rules[3].combinator, ' > ');
    t.equal(tree.selectors[0].rules[3].rules[0].attribute, 'name');
    t.notOk(tree.selectors[0].rules[3].rules[0].value, 'should not have a value');
});

(0, _utilHelpers.test)('more attribute operators', '[href*=test],[href^=test],[href$=test],[href|=test]', function (t, tree) {
    t.plan(4);
    t.equal(tree.selectors[0].rules[0].operator, '*=');
    t.equal(tree.selectors[1].rules[0].operator, '^=');
    t.equal(tree.selectors[2].rules[0].operator, '$=');
    t.equal(tree.selectors[3].rules[0].operator, '|=');
});

(0, _utilHelpers.test)('spaces in attribute selectors', 'h1[  href  *=  "test"  ]', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[1].attribute, '  href  ');
    t.equal(tree.selectors[0].rules[1].operator, '*=');
    t.equal(tree.selectors[0].rules[1].value, '  "test"  ');
});

(0, _utilHelpers.test)('insensitive attribute selector 1', '[href="test" i]', function (t, tree) {
    t.plan(2);
    t.equal(tree.selectors[0].rules[0].value, '"test"');
    t.equal(tree.selectors[0].rules[0].insensitive, true);
});

(0, _utilHelpers.test)('insensitive attribute selector 2', '[href=TEsT i  ]', function (t, tree) {
    t.plan(2);
    t.equal(tree.selectors[0].rules[0].value, 'TEsT');
    t.equal(tree.selectors[0].rules[0].raw.insensitive, ' i  ');
});