'use strict';

var _utilHelpers = require('./util/helpers');

(0, _utilHelpers.test)('column combinator', '.selected||td', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].value, 'selected');
    t.equal(tree.selectors[0].rules[0].combinator, '||');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'td');
});

(0, _utilHelpers.test)('descendant combinator', 'h1 h2', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].value, 'h1');
    t.equal(tree.selectors[0].rules[0].combinator, ' ');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'h2');
});

(0, _utilHelpers.test)('column combinator', '.selected||td', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].value, 'selected');
    t.equal(tree.selectors[0].rules[0].combinator, '||');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'td');
});

(0, _utilHelpers.test)('multiple descendant combinators', 'h1 h2 h3 h4', function (t, tree) {
    t.plan(4);
    t.equal(tree.selectors[0].rules[0].combinator, ' ', 'should have a combinator');
    t.equal(tree.selectors[0].rules[0].rules[0].combinator, ' ', 'should have a combinator');
    t.equal(tree.selectors[0].rules[0].rules[0].rules[0].combinator, ' ', 'should have a combinator');
    t.equal(tree.selectors[0].rules[0].rules[0].rules[0].rules[0].combinator, '', 'should not have a combinator on the last node');
});

(0, _utilHelpers.test)('adjacent sibling combinator', 'h1~h2', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].value, 'h1');
    t.equal(tree.selectors[0].rules[0].combinator, '~');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'h2');
});

(0, _utilHelpers.test)('adjacent sibling combinator (2)', 'h1 ~h2', function (t, tree) {
    t.plan(4);
    t.equal(tree.selectors[0].rules[0].value, 'h1');
    t.equal(tree.selectors[0].rules[0].combinator, ' ~');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'h2');
    t.equal(tree.selectors[0].rules[0].rules[0].combinator, '');
});

(0, _utilHelpers.test)('adjacent sibling combinator (3)', 'h1~ h2', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].value, 'h1');
    t.equal(tree.selectors[0].rules[0].combinator, '~ ');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'h2');
});

(0, _utilHelpers.test)('adjacent sibling combinator (4)', 'h1 ~ h2', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].value, 'h1');
    t.equal(tree.selectors[0].rules[0].combinator, ' ~ ');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'h2');
});

(0, _utilHelpers.test)('adjacent sibling combinator (5)', 'h1~h2~h3', function (t, tree) {
    t.plan(5);
    t.equal(tree.selectors[0].rules[0].value, 'h1');
    t.equal(tree.selectors[0].rules[0].combinator, '~');
    t.equal(tree.selectors[0].rules[0].rules[0].value, 'h2');
    t.equal(tree.selectors[0].rules[0].rules[0].combinator, '~');
    t.equal(tree.selectors[0].rules[0].rules[0].rules[0].value, 'h3');
});

(0, _utilHelpers.test)('multiple combinators', 'h1~h2>h3', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].combinator, '~', 'should have a combinator');
    t.equal(tree.selectors[0].rules[0].rules[0].combinator, '>', 'should have a combinator');
    t.notOk(tree.selectors[0].rules[0].rules[0].rules[0].combinator, 'should not have a combinator on the last node');
});

(0, _utilHelpers.test)('multiple combinators with whitespaces', 'h1 + h2 > h3', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].combinator, ' + ', 'should have a combinator');
    t.equal(tree.selectors[0].rules[0].rules[0].combinator, ' > ', 'should have a combinator');
    t.notOk(tree.selectors[0].rules[0].rules[0].rules[0].combinator, 'should not have a combinator on the last node');
});

(0, _utilHelpers.test)('multiple combinators with whitespaces', 'h1+ h2 >h3', function (t, tree) {
    t.plan(3);
    t.equal(tree.selectors[0].rules[0].combinator, '+ ', 'should have a combinator');
    t.equal(tree.selectors[0].rules[0].rules[0].combinator, ' >', 'should have a combinator');
    t.notOk(tree.selectors[0].rules[0].rules[0].rules[0].combinator, 'should not have a combinator on the last node');
});