'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _selectorsRoot = require('./selectors/root');

var _selectorsRoot2 = _interopRequireDefault(_selectorsRoot);

var _selectorsSelectors = require('./selectors/selectors');

var _selectorsSelectors2 = _interopRequireDefault(_selectorsSelectors);

var _selectorsClassName = require('./selectors/className');

var _selectorsClassName2 = _interopRequireDefault(_selectorsClassName);

var _selectorsComment = require('./selectors/comment');

var _selectorsComment2 = _interopRequireDefault(_selectorsComment);

var _selectorsId = require('./selectors/id');

var _selectorsId2 = _interopRequireDefault(_selectorsId);

var _selectorsTag = require('./selectors/tag');

var _selectorsTag2 = _interopRequireDefault(_selectorsTag);

var _selectorsPseudo = require('./selectors/pseudo');

var _selectorsPseudo2 = _interopRequireDefault(_selectorsPseudo);

var _selectorsAttribute = require('./selectors/attribute');

var _selectorsAttribute2 = _interopRequireDefault(_selectorsAttribute);

var _selectorsUniversal = require('./selectors/universal');

var _selectorsUniversal2 = _interopRequireDefault(_selectorsUniversal);

var _sortAscending = require('./sortAscending');

var _sortAscending2 = _interopRequireDefault(_sortAscending);

var _tokenize = require('./tokenize');

var _tokenize2 = _interopRequireDefault(_tokenize);

var _flatten = require('flatten');

var _flatten2 = _interopRequireDefault(_flatten);

var _indexesOf = require('indexes-of');

var _indexesOf2 = _interopRequireDefault(_indexesOf);

var _uniq = require('uniq');

var _uniq2 = _interopRequireDefault(_uniq);

var Parser = (function () {
    function Parser(input) {
        _classCallCheck(this, Parser);

        this.input = input;
        this.position = 0;
        this.root = new _selectorsRoot2['default']();

        var selectors = new _selectorsSelectors2['default']();
        this.root.append(selectors);

        this.current = selectors;
        this.tokens = (0, _tokenize2['default'])(input);

        return this.parse();
    }

    _createClass(Parser, [{
        key: 'attribute',
        value: function attribute() {
            var attribute = '';
            this.position++;
            while (this.position < this.tokens.length && this.tokens[this.position][0] !== ']') {
                attribute += this.tokens[this.position][1];
                this.position++;
            }
            if (this.position === this.tokens.length && ! ~attribute.indexOf(']')) {
                this.error('Expected a closing square bracket.');
            }
            var parts = attribute.split(/((?:[*~^$|]?)=)/);
            var attr = new _selectorsAttribute2['default']({
                attribute: parts[0],
                operator: parts[1],
                value: parts[2]
            });
            if (parts[2]) {
                var insensitive = parts[2].split(/(\s+i\s*?)$/);
                attr.value = insensitive[0];
                if (insensitive[1]) {
                    attr.insensitive = true;
                    attr.raw.insensitive = insensitive[1];
                }
            }
            this.current.append(attr);
            this.position++;
        }
    }, {
        key: 'combinator',
        value: function combinator() {
            var combinator = '';
            var tokens = this.tokens;
            while (this.position < tokens.length && tokens[this.position][0] === 'space' || tokens[this.position][0] === 'combinator') {
                combinator += this.tokens[this.position][1];
                this.position++;
                if (this.position === tokens.length) {
                    this.error('Unexpected right hand side combinator.');
                }
            }
            var last = this.current.rules[this.current.rules.length - 1];
            if (!last) {
                this.error('Unexpected left hand side combinator.');
            }
            last.combinator = combinator;
            this.current = last;
        }
    }, {
        key: 'comma',
        value: function comma() {
            if (this.position === this.tokens.length - 1) {
                this.error('Unexpected trailing comma.');
            }
            var selectors = new _selectorsSelectors2['default']();
            if (!this.inParens) {
                while (!this.current.parent.selectors) {
                    this.current = this.current.parent;
                }
            }
            this.current.parent.append(selectors);
            this.current = selectors;
            this.position++;
        }
    }, {
        key: 'comment',
        value: function comment() {
            var comment = new _selectorsComment2['default']({ value: this.tokens[this.position][1] });
            this.current.append(comment);
            this.position++;
        }
    }, {
        key: 'error',
        value: function error(message) {
            throw new this.input.error(message);
        }
    }, {
        key: 'pseudo',
        value: function pseudo() {
            var pseudoStr = '';
            while (this.tokens[this.position][0] === ':') {
                pseudoStr += this.tokens[this.position][1];
                this.position++;
            }
            if (this.tokens[this.position][0] === 'word') {
                pseudoStr += this.tokens[this.position][1];
                this.position++;
                var pseudo = new _selectorsPseudo2['default']({ value: pseudoStr });
                if (this.tokens[this.position] && this.tokens[this.position][0] === '(') {
                    var balanced = 1;
                    var inside = [];
                    this.position++;
                    while (this.position < this.tokens.length && balanced) {
                        if (this.tokens[this.position][0] === '(') balanced++;
                        if (this.tokens[this.position][0] === ')') balanced--;
                        if (balanced) {
                            inside.push(this.tokens[this.position]);
                        }
                        this.position++;
                    }
                    if (balanced) {
                        this.error('Expected closing parenthesis.');
                    }
                    var cache = {
                        current: this.current,
                        tokens: this.tokens,
                        position: this.position
                    };
                    this.position = 0;
                    this.tokens = inside;
                    this.current = pseudo.parameters;
                    while (this.position < this.tokens.length) {
                        switch (this.tokens[this.position][0]) {
                            case 'space':
                                this.space();
                                break;
                            case 'comment':
                                this.comment();
                                break;
                            case '[':
                                this.attribute();
                                break;
                            case 'word':
                                this.word();
                                break;
                            case ':':
                                this.pseudo();
                                break;
                            case ',':
                                this.comma();
                                break;
                            case 'combinator':
                                this.combinator();
                                break;
                        }
                        this.position++;
                    }
                    this.current = cache.current;
                    this.tokens = cache.tokens;
                    this.position = cache.position;
                }
                this.current.append(pseudo);
            } else {
                this.error('Unexpected "' + this.tokens[this.position][0] + '" found.');
            }
        }
    }, {
        key: 'space',
        value: function space() {
            var token = this.tokens[this.position];
            // Handle space before and after the selector
            if (this.position === 0 || this.tokens[this.position - 1][0] === ',') {
                this.current.spaces.before = token[1];
                this.position++;
            } else if (this.position === this.tokens.length - 1) {
                this.current.spaces.after = token[1];
                this.position++;
            } else {
                this.combinator();
            }
        }
    }, {
        key: 'universal',
        value: function universal() {
            this.current.append(new _selectorsUniversal2['default']({ value: this.tokens[this.position][1] }));
            this.position++;
        }
    }, {
        key: 'word',
        value: function word() {
            var _this = this;

            var word = this.tokens[this.position][1];
            var hasClass = (0, _indexesOf2['default'])(word, '.');
            var hasId = (0, _indexesOf2['default'])(word, '#');
            if (hasId.length > 1) {
                this.error('Unexpected "#" found.');
            }
            var indices = (0, _sortAscending2['default'])((0, _uniq2['default'])((0, _flatten2['default'])([[0], hasClass, hasId])));
            indices.forEach(function (ind, i) {
                var index = indices[i + 1] || word.length;
                var value = word.slice(ind, index);
                if (~hasClass.indexOf(ind)) {
                    _this.current.append(new _selectorsClassName2['default']({ value: value.slice(1) }));
                } else if (~hasId.indexOf(ind)) {
                    _this.current.append(new _selectorsId2['default']({ value: value.slice(1) }));
                } else {
                    _this.current.append(new _selectorsTag2['default']({ value: value }));
                }
            });
            this.position++;
        }
    }, {
        key: 'parse',
        value: function parse() {
            while (this.position < this.tokens.length) {
                switch (this.tokens[this.position][0]) {
                    case 'space':
                        this.space();
                        break;
                    case 'comment':
                        this.comment();
                        break;
                    case '[':
                        this.attribute();
                        break;
                    case 'word':
                        this.word();
                        break;
                    case ':':
                        this.pseudo();
                        break;
                    case ',':
                        this.comma();
                        break;
                    case '*':
                        this.universal();
                        break;
                    case 'combinator':
                        this.combinator();
                        break;
                }
            }
            return this.root;
        }
    }]);

    return Parser;
})();

exports['default'] = Parser;
module.exports = exports['default'];